# -*- coding:utf-8 -*-
# Created by qinwei on 2017/11/13
#
import atexit
import threading
from time import sleep

from hunters.config import DEFAULT_BROWSER_CONFIG


class BrowserPool(object):
    """
    浏览器池
    用一个池来装连接, 提供给Spider
    """

    def __init__(self, browser_config=DEFAULT_BROWSER_CONFIG, init_count=5, max_count=20, min_idle=5, timeout=3):
        self._browser_config = browser_config
        self._init_count = init_count
        self._max_count = max_count
        self._min_idle = min_idle
        self._timeout = timeout
        self._pool = list()
        self._current_used_count = 0
        self._lock = threading.Lock()

        self.init_pool()

    def init_pool(self):
        for item in range(self._init_count):
            self._pool.append(self._browser_config.new_browser())

    def get(self):
        browser_ = None
        with self._lock:
            if len(self._pool) > 0:
                browser_ = self._pool.pop(0)
            elif self._current_used_count < self._max_count:
                browser_ = self._browser_config.new_browser()
            self._current_used_count += 1
        return browser_

    def return_resource(self, browser):
        self._pool.append(browser)
        self._current_used_count -= 1

    def close_all(self):
        for item in self._pool:
            item.close()
            item.quit()


class ResourcePool(object):
    """
    一个抽象的池, 对实现Factory接口的池
    """

    def __init__(self, factory=None, init_count=5, max_count=20, min_idle=5, timeout=3):
        self._factory = factory
        self._init_count = init_count
        self._max_count = max_count
        self._min_idle = min_idle
        self._timeout = timeout
        self._pool = list()
        self._current_used_count = 0
        self._resource_config = factory.config()
        self._lock = threading.Lock()

        self.init_pool()

    def init_pool(self):
        for item in range(self._init_count):
            self._pool.append(self.new_instance())

        atexit.register(self.close_all)

    def new_instance(self):
        return self._factory.new_instance()

    def get(self):
        while True:
            resource = self._get()
            if resource is None:
                sleep(1)
            else:
                return resource

    def _get(self):
        resource = None
        with self._lock:
            if self._current_used_count >= self._max_count:
                return None

            if len(self._pool) > 0:
                resource = self._pool.pop(0)
            elif self._current_used_count < self._max_count:
                resource = self.new_instance()

            self._current_used_count += 1
            return resource

    def return_resource(self, browser):
        with self._lock:
            self._pool.append(browser)
            self._current_used_count -= 1

    def close_all(self):
        self._factory.destroy_all(self._pool)



# -*- coding:utf-8 -*-
# Created by qinwei on 2017/11/17
#

from hunters.browser import MiniBrowser, ViewBrowser
from hunters.factory import Factory


class MiniBrowserFactory(Factory):
    """"""

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def new_instance(self):
        return MiniBrowser(browser_config=self.config())

    def destroy(self, obj):
        """ """

    def destroy_all(self, pool_list=None):
        """ """


class ViewBrowserFactory(Factory):
    """"""

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def new_instance(self):
        return ViewBrowser(browser_config=self.config())

    def destroy(self, obj):
        obj.close()
        obj.quit()

    def destroy_all(self, pool_list=None):
        for obj in pool_list:
            obj.close()
            obj.quit()

# bin_path = r"C:\Program Files (x86)\Google\Chrome\Application\chrome.exe {}".format(" ".join(chromeFlags))


# bin = r"C:\Program Files (x86)\Google\Chrome\Application\chrome.exe --remote-debugging-port=9222"



# -*- coding:utf-8 -*-
# Created by qinwei on 2017/11/15
#
import unittest
from time import time

import logging
from selenium import webdriver

from hunters.browser import ViewBrowser

URL_LIST = ['http://www.jianshu.com/p/8b1a48833b94', 'http://www.jianshu.com/p/79df4bc92800', 'http://www.jianshu.com/p/84dd7caeb990',
            'http://www.jianshu.com/p/d1d2f1e8e2e4', 'http://www.jianshu.com/p/26507d708306', 'http://www.jianshu.com/p/6cfa6df9df68',
            'http://www.jianshu.com/p/45fae71ad26b', 'http://www.jianshu.com/p/aeb1eea885ae', 'http://www.jianshu.com/p/0fa1d71f2650',
            'http://www.jianshu.com/p/395c8b15d385', 'http://www.jianshu.com/p/9ddcf949676f', 'http://www.jianshu.com/p/c0ca2c230149',
            'http://www.jianshu.com/p/7813463e4772', 'http://www.jianshu.com/p/437c2ae96396', 'http://www.jianshu.com/p/11a958b4ea26',
            'http://www.jianshu.com/p/3ebd923f63b7', 'http://www.jianshu.com/p/8b14bbcbf886', 'http://www.jianshu.com/p/d29ce5cb9a0c',
            'http://www.jianshu.com/p/a8c936689233', 'http://www.jianshu.com/p/b81562687f3e']

NEW_LIST = ['http://news.xinhuanet.com/world/2017-11/15/c_129741004.htm', 'http://news.xinhuanet.com/politics/leaders/2017-11/14/c_1121953758.htm',
            'http://world.huanqiu.com/special/2017APEC/index.html?from=bdwz', 'http://news.xhby.net/system/2017/11/14/030765790.shtml',
            'http://news.xinhuanet.com/politics/2017-11/15/c_1121959898.htm', 'http://news.xinhuanet.com/world/2017-11/14/c_1121955602.htm',
            'http://news.cctv.com/2017/11/14/ARTIQvcSOAjXcaZ5OIFtJf4D171114.shtml', 'http://news.ifeng.com/mainland/special/xsdjj/?_zbs_baidu_news',
            'http://www.ddcpc.cn/2017/jr_1113/113598.html', 'http://news.gmw.cn/2017-11/15/content_26790042.htm',
            'http://www.am774.com/activity/laoshezhangdegushi.shtml', 'http://news.youth.cn/gn/201711/t20171115_11012409.htm',
            'http://world.huanqiu.com/article/2017-11/11380352.html?from=bdwz ', 'http://china.huanqiu.com/article/2017-11/11380050.html?from=bdwz',
            'http://news.ifeng.com/a/20171115/53265266_0.shtml?_zbs_baidu_news', 'http://china.huanqiu.com/article/2017-11/11379982.html?from=bdwz',
            'http://china.huanqiu.com/article/2017-11/11379260.html?from=bdwz', 'http://news.ifeng.com/a/20171115/53264294_0.shtml?_zbs_baidu_news',
            'https://baijia.baidu.com/topic?topic_id=718', 'http://xinwen.eastday.com/a/xjump.html?id=171115121048639',
            'http://china.huanqiu.com/article/2017-11/11379367.html?from=bdwz', 'http://news.china.com/domestic/945/20171115/31665340.html',
            'http://china.huanqiu.com/article/2017-11/11379874.html?from=bdwz?', 'http://china.huanqiu.com/article/2017-11/11380275.html?from=bdwz ',
            'http://world.huanqiu.com/exclusive/2017-11/11380341.html?from=bdwz', 'http://news.ifeng.com/a/20171115/53266932_0.shtml?_zbs_baidu_news',
            'http://world.huanqiu.com/exclusive/2017-11/11379416.html?from=bdwz?',
            'http://news.ifeng.com/a/20171115/53270527_0.shtml?_zbs_baidu_news', 'http://xinwen.eastday.com/a/xjump.html?id=171115130117314',
            'http://news.ifeng.com/a/20171115/53265948_0.shtml?_zbs_baidu_news', 'http://society.huanqiu.com/article/2017-11/11379721.html?from=bdwz',
            'http://finance.china.com.cn/hz/sh/2345/20171115/18061.shtml', 'http://society.huanqiu.com/article/2017-11/11380111.html?from=bdwz',
            'http://finance.china.com.cn/hz/sh/2345/20171115/18060.shtml', 'http://finance.china.com.cn/hz/sh/2345/20171115/18066.shtml',
            'http://xinwen.eastday.com/a/xjump.html?id=171115123025402', 'http://news.ifeng.com/a/20171115/53273421_0.shtml?_zbs_baidu_news',
            'http://china.huanqiu.com/article/2017-11/11379326.html?from=bdwz', 'http://news.china.com/socialgd/10000169/20171115/31666218.html',
            'http://finance.china.com.cn/hz/sh/2345/20171115/18040.shtml', 'http://society.huanqiu.com/article/2017-11/11379497.html?from=bdwz',
            'http://news.ifeng.com/a/20171115/53266049_0.shtml?_zbs_baidu_news']


class seleniumTest(unittest.TestCase):
    def setUp(self):
        self.driver = webdriver.PhantomJS(executable_path=r"D:\data\apps\phantomjs-2.1.1-windows\bin\phantomjs.exe",
                                          service_args=["--disk-cache=true", "--load-images=true", "--web-security=false"])
        option = webdriver.ChromeOptions()
        option.add_argument('--allow-running-insecure-content')
        option.add_argument('--disable-web-security')
       # option.add_argument("--headless")
        # option.add_argument("--disable-gpu")
        option.add_argument("--disable-background-networking")
        option.add_argument("--disable-client-side-phishing-detection")

        #prefs = {"profile.managed_default_content_settings.images": 2}  # disable image
        #option.add_experimental_option("prefs", prefs)

        self.driver = webdriver.Chrome(executable_path=r"D:\webdriver\chromedriver.exe", chrome_options=option)
        # self.driver = ViewBrowser()
        self.driver.get("http://baidu.com")

    def testEle(self):
        driver = self.driver
        MAX = 50
        for url in range(100):
            start = time()
            driver.get("http://baidu.com")
            #driver.page_source
            second = time() - start
            print("%.3f" % second)


if __name__ == "__main__":
    unittest.main()



# -*- coding:utf-8 -*-
# Created by qinwei on 2017/9/11
import logging

from urllib3.exceptions import ReadTimeoutError

logger = logging.getLogger("hunters.exception")


class OverLimitErr(Exception):
    """ Add Url Over limit """


class SpiderExceptionListener(object):
    """
    总的Exception控制器
    """

    def __init__(self):
        self._exception_handler = []
        self.init_exception()

    def init_exception(self):
        def show_exception(ex, input_param, position):
            msg = "{}, {}, {}".format(ex, input_param, position)
            if isinstance(ex, ReadTimeoutError):
                logger.warning("{} {}".format("ReadTimeoutError", msg))
            else:
                logger.exception(msg)

        self.add_listener(show_exception)

    def add_listener(self, ex):
        """ add a exception listener


        :param ex: def handler(ex, input_param, position)
        :return:
        """
        if not callable(ex):
            raise ValueError("{} not callable")
        self._exception_handler.append(ex)

    def handle_exception(self, ex, input_params, pos):
        for handler in self._exception_handler:
            handler(ex, input_params, pos)


spider_exception_listener = SpiderExceptionListener()



# -*- coding:utf-8 -*-
# Created by qinwei on 2017/11/23
#
import unittest
from time import sleep

import pychrome

from hunters.chrome import open_chrome

NEW_LIST = ['http://news.xinhuanet.com/world/2017-11/15/c_129741004.htm',
            'http://news.xinhuanet.com/politics/leaders/2017-11/14/c_1121953758.htm',
            'http://world.huanqiu.com/special/2017APEC/index.html?from=bdwz',
            'http://news.xhby.net/system/2017/11/14/030765790.shtml',
            'http://news.xinhuanet.com/politics/2017-11/15/c_1121959898.htm',
            'http://news.xinhuanet.com/world/2017-11/14/c_1121955602.htm',
            'http://news.cctv.com/2017/11/14/ARTIQvcSOAjXcaZ5OIFtJf4D171114.shtml',
            'http://news.ifeng.com/mainland/special/xsdjj/?_zbs_baidu_news',
            'http://www.ddcpc.cn/2017/jr_1113/113598.html', 'http://news.gmw.cn/2017-11/15/content_26790042.htm',
            'http://www.am774.com/activity/laoshezhangdegushi.shtml',
            'http://news.youth.cn/gn/201711/t20171115_11012409.htm',
            'http://world.huanqiu.com/article/2017-11/11380352.html?from=bdwz ',
            'http://china.huanqiu.com/article/2017-11/11380050.html?from=bdwz',
            'http://news.ifeng.com/a/20171115/53265266_0.shtml?_zbs_baidu_news',
            'http://china.huanqiu.com/article/2017-11/11379982.html?from=bdwz',
            'http://china.huanqiu.com/article/2017-11/11379260.html?from=bdwz',
            'http://news.ifeng.com/a/20171115/53264294_0.shtml?_zbs_baidu_news',
            'https://baijia.baidu.com/topic?topic_id=718', 'http://xinwen.eastday.com/a/xjump.html?id=171115121048639',
            'http://china.huanqiu.com/article/2017-11/11379367.html?from=bdwz',
            'http://news.china.com/domestic/945/20171115/31665340.html',
            'http://china.huanqiu.com/article/2017-11/11379874.html?from=bdwz?',
            'http://china.huanqiu.com/article/2017-11/11380275.html?from=bdwz ',
            'http://world.huanqiu.com/exclusive/2017-11/11380341.html?from=bdwz',
            'http://news.ifeng.com/a/20171115/53266932_0.shtml?_zbs_baidu_news',
            'http://world.huanqiu.com/exclusive/2017-11/11379416.html?from=bdwz?',
            'http://news.ifeng.com/a/20171115/53270527_0.shtml?_zbs_baidu_news',
            'http://xinwen.eastday.com/a/xjump.html?id=171115130117314',
            'http://news.ifeng.com/a/20171115/53265948_0.shtml?_zbs_baidu_news',
            'http://society.huanqiu.com/article/2017-11/11379721.html?from=bdwz',
            'http://finance.china.com.cn/hz/sh/2345/20171115/18061.shtml',
            'http://society.huanqiu.com/article/2017-11/11380111.html?from=bdwz',
            'http://finance.china.com.cn/hz/sh/2345/20171115/18060.shtml',
            'http://finance.china.com.cn/hz/sh/2345/20171115/18066.shtml',
            'http://xinwen.eastday.com/a/xjump.html?id=171115123025402',
            'http://news.ifeng.com/a/20171115/53273421_0.shtml?_zbs_baidu_news',
            'http://china.huanqiu.com/article/2017-11/11379326.html?from=bdwz',
            'http://news.china.com/socialgd/10000169/20171115/31666218.html',
            'http://finance.china.com.cn/hz/sh/2345/20171115/18040.shtml',
            'http://society.huanqiu.com/article/2017-11/11379497.html?from=bdwz',
            'http://news.ifeng.com/a/20171115/53266049_0.shtml?_zbs_baidu_news']


class TestChromePool(unittest.TestCase):
    def setUp(self):
        open_chrome(9223, headless=False)

    def test_main(self):
        def responseReceived(**kwargs):
            request_id = kwargs.get("requestId")
            print(request_id, kwargs.get("type"), kwargs.get("response").get("url"))
            print(tab.Network.getResponseBody(requestId=request_id))  # {"body":"",  "base64Encoded": True/False }
            # Network.getResponseBody

        browser = pychrome.Browser(url="http://127.0.0.1:{}".format(9223))
        # tab = browser.new_tab("chrome://settings/content/images")
        tab = browser.new_tab("about:blank")
        tab.Network.enable()
        tab.Network.responseReceived = responseReceived
        tab.Network.setBlockedURLs(urls=[".css", ".jpg", ".png", ".ico", ".gif", ".jpeg", "img="])
        tab.Network.setDataSizeLimitsForTest(maxTotalSize=1024 * 1024, maxResourceSize=300 * 1024)
        tab.Page.navigate(url="http://baidu.com")
        try:
            # 实验性的接口测试通过, 可以动态调整windows大小, GUI模式下才行, Headless不行
            bounds = tab.Browser.getWindowBounds(windowId=1)

            bounds = {'height': 1100,
                      'left': 502,
                      'top': 22,
                      'width': 1440,
                      'windowState': 'normal'}

            tab.Browser.setWindowBounds(windowId=1, bounds=bounds)
        except Exception as e:
            print(e)

        data = tab.Network.getAllCookies()
        tab.Page.navigate(url="http://www.sohu.com/a/205949961_255783?_f=index_news_18?pvid=ab2a3f3fa09b29b9")
        chromedata = tab.Runtime.evaluate(expression="chrome.contentSettings", returnByValue=True, awaitPromise=True)
        print(chromedata)
        # tab.Browser.close()
        # print(data)
        while True:
            sleep(10)



# -*- coding:utf-8 -*-
# Created by qinwei on 2017/8/8
import logging

from hunters.browser import ViewBrowser
from hunters.config import BrowserConfig

logging.basicConfig(level=logging.DEBUG)
# v = ViewBrowser()
canary = r"C:\Users\ADMIN\AppData\Local\Google\Chrome SxS\Application\chrome.exe"
chrome = r"C:\Program Files (x86)\Google\Chrome\Application\chrome.exe"
c = BrowserConfig(execute_bin=canary, headless=False)
domain = "duba.com"
view = ViewBrowser(browser_config=c)
r = view.get("http://{}".format(domain), timeout=5)
for i in range(3):
    r.scroll_by(0, 2000)
    r.wait(2.5)

r.wait(10)
r.screenshot_as_png("{}.png".format(domain), full_page=True)

r.wait(10)
print("OK")



# -*- coding:utf-8 -*-
# Created by qinwei on 2017/11/17
#
import re
import threading
from base64 import b64decode
from time import sleep

from hunters.factory import ResourcePool, ChromeTabDevToolFactory
from test.test_chromepool import NEW_LIST

COUNT = 20
chrome_factory = ChromeTabDevToolFactory(headless=True)
# chrome_factory = ChromeFactory(headless=True)
pool = ResourcePool(factory=chrome_factory, init_count=COUNT)


def get_page(url_):
    tab = pool.get()
    tab.Page.enable()
    tab.Page.setAdBlockingEnabled(enabled=True)
    result = tab.Page.navigate(url=url_, timeout=10)
    pool.return_resource(tab)


def scroll_by():
    tab = pool.get()
    print(tab.Runtime.evaluate(expression="window.scrollBy(0, 3000)", returnByValue=False, awaitPromise=True))
    pool.return_resource(tab)


def get_title():
    tab = pool.get()
    print(tab.Runtime.evaluate(expression="document.title", returnByValue=False, awaitPromise=True))
    pool.return_resource(tab)


def set_value(value):
    tab = pool.get()
    print(tab.Runtime.evaluate(expression="data_url={}".format(value), returnByValue=False, awaitPromise=True))
    pool.return_resource(tab)


def sceenshot():
    print("start screenshot")
    # XXX 不能带clip, 并且需要fromSurface=True headless才能截屏!!!
    tab = pool.get()
    sleep(2)
    # bounds = {'height': 800,
    #           'left': 0,
    #           'top': 0,
    #           'width': 800,
    #           'windowState': 'normal'}
    #
    # tab.Browser.setWindowBounds(windowId=1, bounds=bounds)  # XXX headless模式下找不到setWindowBounds
    width, height = 1280, 2000
    tab.Target.activateTarget(targetId=tab.id)
    data = tab.Emulation.setDeviceMetricsOverride(width=width, height=height, deviceScaleFactor=0, mobile=False,
                                                  fitWindow=True)
    print(data)
    data = tab.Emulation.setVisibleSize(width=width, height=height)
    print(data)
    result = tab.Page.captureScreenshot(format="png", quality=1, fromSurface=True, fullPage=True)
    base64 = result.get('data', "")
    data = tab.Runtime.evaluate(expression="document.title", returnByValue=True, awaitPromise=True)
    print(data)
    filename = data.get("result").get("value")
    unsafe_code = re.compile("[^\u4e00-\u9fa5a-z0-9]+", re.I)
    save_file = "D:/{}.jpg".format(unsafe_code.sub("", filename))
    print(save_file)
    with open(save_file, "wb+") as f:
        f.write(b64decode(base64))
    pool.return_resource(tab)


def main():
    for i in NEW_LIST[:COUNT]:
        get_page(i)

    sleep(3)

    threads = []
    for i in range(COUNT):
        try:
            t = threading.Thread(target=sceenshot, args=())
            threads.append(t)
            # scroll_by()
            # get_title()
            # set_value(i)
            # sceenshot()  # XXX 执行截图需要tab必须是activity的.
            # sleep(1)
        except Exception as e:
            print(e)

    for t in threads:
        t.start()

    for t in threads:
        t.join()

    sleep(20)

    pool.close_all()


main()

# for i in range(10):
#     print(tab.Runtime.evaluate(expression="location.href", returnByValue=False, awaitPromise=True))
